//
//  EaseFunctionsViewController.m
//  EaseFunctions
//
//  Created by Niall McCormack on 13/07/2011.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>
#import "EaseFunctionsViewController.h"
#import "MEEaseFunctionBuilder.h"


@interface EaseFunctionsViewController ()

@property(nonatomic, assign) BOOL isPickerShowing;
@property(nonatomic, retain) NSArray *tweenArray;
@property(nonatomic, retain) NSArray *easeArray;
@property(nonatomic, retain) NSArray *typeArray;
@property(nonatomic, retain) NSArray *easeTypeArray;
@property(nonatomic, retain) CAMediaTimingFunction *timingFunction;
@property(nonatomic, assign) NSUInteger easeIndex;
@property(nonatomic, assign) NSUInteger typeIndex;
@property(nonatomic, assign) CGFloat duration;
@property(nonatomic, retain) NSArray *durationTextArray;
@property(nonatomic, retain) NSArray *durationArray;

-(void)viewTapped:(UITapGestureRecognizer *)tapGesture;
-(void)animatePickerIn:(BOOL)yesForIn;
-(void)updateTimingFunction;

@end

@implementation EaseFunctionsViewController

@synthesize puck = _puck;
@synthesize picker = _picker;
@synthesize isPickerShowing = _isPickerShowing;
@synthesize tweenArray = _tweenArray;
@synthesize easeTypeArray = _easeTypeArray;
@synthesize easeArray = _easeArray;
@synthesize typeArray = _typeArray;
@synthesize button = _button;
@synthesize timingFunction = _timingFunction;
@synthesize containerView = _containerView;
@synthesize easeIndex = _easeIndex;
@synthesize typeIndex = _typeIndex;
@synthesize duration = _duration;
@synthesize durationTextArray = _durationTextArray;
@synthesize durationArray = _durationArray;

- (id)initWithCoder:(NSCoder *)coder {
    self = [super initWithCoder:coder];
    
    if (self) {
        
        self.easeArray = [[NSArray alloc] initWithObjects:@"Sine", @"Quad", @"Cubic", @"Quart", @"Quint", @"Expo", @"Circ", @"Back", nil];
        self.typeArray = [[NSArray alloc] initWithObjects:@"In", @"Out", @"In Out", nil];
        
        self.easeTypeArray = [[NSArray alloc] initWithObjects:[NSNumber numberWithInt:EaseTypeSineIn], [NSNumber numberWithInt:EaseTypeSineOut], [NSNumber numberWithInt:EaseTypeSineInOut],
                              [NSNumber numberWithInt:EaseTypeQuadIn], [NSNumber numberWithInt:EaseTypeQuadOut], [NSNumber numberWithInt:EaseTypeQuadInOut],
                              [NSNumber numberWithInt:EaseTypeCubicIn], [NSNumber numberWithInt:EaseTypeCubicOut], [NSNumber numberWithInt:EaseTypeCubicInOut],
                              [NSNumber numberWithInt:EaseTypeQuartIn], [NSNumber numberWithInt:EaseTypeQuartOut], [NSNumber numberWithInt:EaseTypeQuartInOut],
                              [NSNumber numberWithInt:EaseTypeQuintIn], [NSNumber numberWithInt:EaseTypeQuintOut], [NSNumber numberWithInt:EaseTypeQuintInOut],
                              [NSNumber numberWithInt:EaseTypeExpoIn], [NSNumber numberWithInt:EaseTypeExpoOut], [NSNumber numberWithInt:EaseTypeExpoInOut],
                              [NSNumber numberWithInt:EaseTypeCircIn], [NSNumber numberWithInt:EaseTypeCircOut], [NSNumber numberWithInt:EaseTypeCircInOut],
                              [NSNumber numberWithInt:EaseTypeBackIn], [NSNumber numberWithInt:EaseTypeBackOut], [NSNumber numberWithInt:EaseTypeBackInOut], 
                              nil];
        self.durationTextArray = [[NSArray alloc] initWithObjects:@"0.25 seconds", @"0.5 seconds", @"0.75 seconds", @"1 second", @"2 seconds", nil];
        self.durationArray = [[NSArray alloc] initWithObjects:[NSNumber numberWithFloat:0.25], [NSNumber numberWithFloat:0.5], [NSNumber numberWithFloat:0.75], [NSNumber numberWithFloat:1.0], [NSNumber numberWithFloat:2.0], nil];
        
    }
    
    return self;
}


- (void)dealloc
{
    [_picker release];
    _picker = nil;
    
    [_puck release];
    _puck = nil;
    
    [_button release];
    _button = nil;
    
    [_containerView release];
    _containerView = nil;
    
    [_tweenArray release];
    _tweenArray = nil;
    
    [_easeArray release];
    _easeArray = nil;
    
    [_typeArray release];
    _typeArray = nil;
    
    [_easeTypeArray release];
    _easeTypeArray = nil;
    
    [_timingFunction release];
    _timingFunction = nil;
    
    [_durationTextArray release];
    _durationTextArray = nil;
    
    [_durationArray release];
    _durationArray = nil;
    
    
    [super dealloc];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle
// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad
{
    [super viewDidLoad];
    
    // create puck
    self.puck = [CALayer layer];
    self.puck.backgroundColor = [UIColor redColor].CGColor;
    self.puck.frame = CGRectMake(0, 0, 100, 100);
    
    [self.containerView.layer addSublayer:self.puck];
    
    // add gestures
    UITapGestureRecognizer *tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(viewTapped:)];
    [tapGesture setNumberOfTapsRequired:1];
    [tapGesture setNumberOfTouchesRequired:1];
    
    [self.containerView addGestureRecognizer:tapGesture];
    
    
    // setup picker
    [self.picker setFrame:(CGRect){{0, self.view.frame.size.height}, self.picker.frame.size}];
//    self.picker.layer.anchorPoint = CGPointZero;
    
    [self.picker selectRow:1 inComponent:0 animated:NO];
    [self.picker selectRow:2 inComponent:1 animated:NO];
    [self.picker selectRow:0 inComponent:2 animated:NO];
    
    // define initial easing function
    self.easeIndex = 1;
    self.typeIndex = 2;
    self.duration = [(NSNumber *)[self.durationArray objectAtIndex:0] floatValue];
    
    [self updateTimingFunction];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

-(void)animatePickerIn:(BOOL)yesForIn
{
    CGPoint position;
    
    if( yesForIn ) {
        position = CGPointMake(0, self.view.frame.size.height - self.picker.frame.size.height);
    } else {
        position = CGPointMake(0, self.view.frame.size.height);
    }
    
    
    CALayer *pickerLayer = self.picker.layer;
    [pickerLayer setDelegate:self];
    
    [CATransaction setAnimationDuration:0.25];
    [CATransaction setAnimationTimingFunction:self.timingFunction];
    
    CGRect pickerFrame = pickerLayer.frame;
    pickerFrame.origin = position;
    pickerLayer.frame = pickerFrame;
    
    [pickerLayer setDelegate:nil];
}

-(void)updateTimingFunction
{
    NSNumber *easeType = [self.easeTypeArray objectAtIndex:(self.easeIndex * 3) + self.typeIndex];
    self.timingFunction = [MEEaseFunctionBuilder functionWithType:[easeType intValue]];
}

#pragma mark - Events
-(IBAction)buttonPressed:(UIButton *)button
{
    if( self.isPickerShowing ) {
        [self animatePickerIn:NO];
    } else {
        [self animatePickerIn:YES];
    }
    
    self.isPickerShowing = !self.isPickerShowing;
}

-(void)viewTapped:(UITapGestureRecognizer *)tapGesture
{
    if( self.isPickerShowing ) {
        
        [self animatePickerIn:NO];
        self.isPickerShowing = !self.isPickerShowing;
        
        return;
    }
    
    CGPoint point = [tapGesture locationOfTouch:0 inView:self.containerView];
    
    [CATransaction setAnimationDuration:self.duration];
    [CATransaction setAnimationTimingFunction:self.timingFunction];
    self.puck.position = point;
}

#pragma mark - UIPickerView Delegate and Datasource Methods
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
    return 3;
}

- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
    switch (component) {
        case 0:
            return [self.easeArray count];
            break;
        case 1:
            return [self.typeArray count];
            break;
        case 2:
            return [self.durationTextArray count];
            break;
        default:
            return 0;
            break;
    }
}

- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component
{
    switch (component) {
        case 0:
            return [self.easeArray objectAtIndex:row];
            break;
        case 1:
            return [self.typeArray objectAtIndex:row];
            break;
        case 2:
            return [self.durationTextArray objectAtIndex:row];
            break;
        default:
            return @"";
            break;
    }
}

- (void)pickerView:(UIPickerView *)pickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component
{
    switch (component) {
        case 0:
            self.easeIndex = row;
            [self updateTimingFunction];
            break;
        case 1:
            self.typeIndex = row;
            [self updateTimingFunction];
            break;
        case 2:
            self.duration = [(NSNumber *)[self.durationArray objectAtIndex:row] floatValue];
            break;
    }
}

@end